package com.yihu.hos.arbiter.services;

import com.yihu.hos.arbiter.configuration.ArbiterServerConfiguration;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.constant.EndPointConstant;
import com.yihu.hos.web.framework.constant.MycatConstant;
import com.yihu.hos.web.framework.constant.SSHConstant;
import com.yihu.hos.web.framework.constant.ServiceFlowConstant;
import com.yihu.hos.web.framework.util.HostUtil;
import org.apache.camel.CamelContext;
import org.apache.camel.ProducerTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.zbus.broker.ZbusBroker;
import org.zbus.mq.Consumer;
import org.zbus.net.http.Message;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Airhead
 * @since 2016/12/22.
 */
@Service
public class ProxyService {
    private static final Logger logger = LoggerFactory.getLogger(ProxyService.class);

    private CamelContext camelContext;
    private ArbiterServerConfiguration configuration;
    private ZbusBroker zbusBroker;

    @Autowired
    public void setZbusBroker(ZbusBroker zbusBroker) {
        this.zbusBroker = zbusBroker;
    }

    @Autowired
    public void setCamelContext(CamelContext camelContext) {
        this.camelContext = camelContext;
    }

    @Autowired
    public void setConfiguration(ArbiterServerConfiguration configuration) {
        this.configuration = configuration;
    }

    public void start() {
        new ServiceFlow().server();
        new Mycat().start();
        new Shell().start();
    }

    public class ServiceFlow {
        private Consumer consumer;

        public void handle(Message message, Consumer consumer) {
            Map<String, Object> header = new HashMap<>();
            header.put("event", message.getHead("event"));

            ProducerTemplate producerTemplate = camelContext.createProducerTemplate();
            producerTemplate.sendBodyAndHeaders(ServiceFlowConstant.CAMEL_ENDPOINT, message.getBodyString(), header);
        }

        public void server() {
            if (zbusBroker == null) {
                logger.error("zbusBroker is null");
                return;
            }

            try {
                consumer = new Consumer(zbusBroker, ServiceFlowConstant.ZBUS_MQ + "@" + configuration.getTenant());
                consumer.start(this::handle);
            } catch (IOException e) {
                e.printStackTrace();
                logger.error(e.getMessage());
            }
        }

        @Override
        protected void finalize() throws Throwable {
            if (consumer != null) {
                consumer.close();
            }

            super.finalize();
        }
    }

    public class Mycat {
        private Consumer consumer;

        public void handle(Message message, Consumer consumer) {
            Map<String, Object> header = new HashMap<>();
            header.put("event", message.getHead("event"));

            ProducerTemplate producerTemplate = camelContext.createProducerTemplate();
            producerTemplate.sendBodyAndHeaders(MycatConstant.CAMEL_ENDPOINT, message.getBodyString(), header);
        }

        public void start() {
            if (zbusBroker == null) {
                logger.error("zbusBroker is null");
                return;
            }

            try {
                String ip = HostUtil.getAddress();
                consumer = new Consumer(zbusBroker, MycatConstant.ZBUS_MQ + "@" + configuration.getTenant()+ip);
                consumer.start(this::handle);
            } catch (IOException e) {
                e.printStackTrace();
                logger.error(e.getMessage());
            }
        }

        @Override
        protected void finalize() throws Throwable {
            if (consumer != null) {
                consumer.close();
            }

            super.finalize();
        }
    }


    public class Shell {
        private Consumer consumer;

        public void handle(Message message, Consumer consumer) {
            Map<String, Object> header = new HashMap<>();
            header.put("event", message.getHead("event"));

            ProducerTemplate producerTemplate = camelContext.createProducerTemplate();
            producerTemplate.sendBodyAndHeaders(SSHConstant.CAMEL_ENDPOINT, message.getBodyString(), header);
        }

        public void start() {
            if (zbusBroker == null) {
                logger.error("zbusBroker is null");
                return;
            }
            try {
                consumer = new Consumer(zbusBroker, SSHConstant.SHELL_REQUEST + "@" + configuration.getTenant());
                consumer.start(this::handle);
            } catch (IOException e) {
                e.printStackTrace();
                logger.error(e.getMessage());
            }
        }

        @Override
        protected void finalize() throws Throwable {
            if (consumer != null) {
                consumer.close();
            }

            super.finalize();
        }
    }

    public class EndPoint {
        private Consumer consumer;

        public void handle(Message message, Consumer consumer) {
            Map<String, Object> header = new HashMap<>();
            header.put("event", message.getHead("event"));

            ProducerTemplate producerTemplate = camelContext.createProducerTemplate();
            producerTemplate.sendBodyAndHeaders(SSHConstant.CAMEL_ENDPOINT, message.getBodyString(), header);
        }

        public void start() {
            if (zbusBroker == null) {
                logger.error("zbusBroker is null");
                return;
            }
            try {
                consumer = new Consumer(zbusBroker, EndPointConstant.ZBUS_MQ + "@" + configuration.getTenant());
                consumer.start(this::handle);
            } catch (IOException e) {
                e.printStackTrace();
                logger.error(e.getMessage());
            }
        }

        @Override
        protected void finalize() throws Throwable {
            if (consumer != null) {
                consumer.close();
            }

            super.finalize();
        }
    }
}
